/**
 * 
 */
package edu.wisc.uwss.configuration.preauth;

import javax.servlet.Filter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.web.authentication.preauth.AbstractPreAuthenticatedProcessingFilter;

import edu.wisc.uwss.preauth.PreAuthLogoutSuccessHandler;
import edu.wisc.uwss.preauth.UWUserDetailsAuthenticationFilter;

/**
 * {@link WebSecurityConfigurerAdapter} for the "preauth" {@link Profile}.
 * The intended use by others is to extend this class, Override the {@link #configure(HttpSecurity)} method (invoking super - this implementation), 
 * and adding whatever additional functionality on {@link HttpSecurity} is required. Most often, you will only need to detail your URI
 * security (using {@link HttpSecurity#authorizeRequests()}).
 * 
 * @author Nicholas Blair
 */
@Configuration
@EnableWebSecurity
@Profile("preauth")
public class WebSecurityConfiguration extends WebSecurityConfigurerAdapter {
  
  protected final Logger logger = LoggerFactory.getLogger(this.getClass());
  /**
  /**
   * {@inheritDoc}
   * 
   * <ul>
   * <li>Adds {@link UWUserDetailsAuthenticationFilter} to the chain, see {@link #authenticationFilter()}.</li>
   * <li>Enables {@link HttpSecurity#formLogin()} and {@link HttpSecurity#httpBasic()} for demo users.</li>
   * <li>Wires in {@link PreAuthLogoutSuccessHandler} to logout.</li>
   * </ul>
   */
  @Override
  protected void configure(HttpSecurity http) throws Exception { 
    logger.debug("enter WebSecurityConfigurerAdapter#configure for local-users");  
    http
      .addFilterBefore(authenticationFilter(), AbstractPreAuthenticatedProcessingFilter.class)
      .logout()
        .logoutSuccessHandler(new PreAuthLogoutSuccessHandler())
        .deleteCookies("JSESSIONID")
        .invalidateHttpSession(true);
    
    // CSRF protection is disabled, as without it /logout requires POST
    http
      .csrf().disable();
  }

  /**
   * 
   * @return an instance of {@link UWUserDetailsAuthenticationFilter}
   * @throws Exception 
   */
  @Bean
  public Filter authenticationFilter() throws Exception {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      filter.setAuthenticationManager(authenticationManager());
      return filter;
  }

  /**
   * Exposes the internal {@link HttpSecurity} as a bean so deployers can customize it further.
   * 
   * @return
   * @throws Exception
   */
  @Bean
  public HttpSecurity httpSecurity() throws Exception {
    return getHttp();
  }
}
