/**
 * 
 */
package edu.wisc.uwss.local;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import edu.wisc.uwss.UWUserDetailsImpl;

/**
 * Implementation of {@link UserDetailsService} intended to provide local "demo" accounts.
 * This implementation is not intended for production use; it is primarily intended to support
 * development and/or test environments. If activating in a test environment, make sure that other
 * defense (firewall) is in place, as typically the demo users have weak, non-compliant passwords.
 * 
 * @author Nicholas Blair
 */
public class LocalUserDetailsServiceImpl implements UserDetailsService {

  private final Map<String, UWUserDetailsImpl> users = new ConcurrentHashMap<>();

  /**
   * Populate the internal map of {@link UWUserDetailsImpl} via the entries in the
   * injected {@link Properties}.
   * 
   * The format of the properties is as follows (key=value):
   * <pre>
   * username=password,fullName,emailAddress[,membership]
   * </pre>
   * 
   * To add a UDDS membership to the demo user, the String should look like a UDDS.
   * 
   * @param properties
   */
  @Autowired
  public void setDemoUsers(@Qualifier("demo-users") Properties properties) {
    for(Entry<Object, Object> entry: properties.entrySet()) {
      String[] values = ((String) entry.getValue()).split(",");
      String username = (String) entry.getKey();

      final UWUserDetailsImpl user;
      if(values.length > 4) {
        String[] uddsValues = Arrays.copyOfRange(values, 4, values.length);
        List<String> membership = new ArrayList<>();
        for(String v : uddsValues) {
          membership.add(v); 
        }
        user = new UWUserDetailsImpl(values[1], username, values[0], values[2], values[3], membership);
      } else {
        user = new UWUserDetailsImpl(values[1], username, values[0], values[2], values[3]);
      }
      user.setSource("local-users");
      users.put(username, user);
    }
  }
  /**
   * Add a user. Does nothing if the argument is null or the {@link UWUserDetailsImpl#getUsername()} is blank.
   * 
   * @param userDetails the user to add.
   */
  void addDemoUser(UWUserDetailsImpl userDetails) {
    if(userDetails != null && StringUtils.isNotBlank(userDetails.getUsername())) {
      users.put(userDetails.getUsername(), userDetails);
    }
  }
  /**
   * {@inheritDoc}
   * 
   * An important aspect of this method is the fact that a clone of the {@link UWUserDetailsImpl} stored
   * in the internal map is returned, NOT a reference to the instance in the map.
   * Spring Security nulls out the password field; if we were to return the instance in the map, we would
   * not be able to log in as that user again (as the password field is damaged).
   */
  @Override
  public UWUserDetailsImpl loadUserByUsername(String username)
      throws UsernameNotFoundException {
    UWUserDetailsImpl user = users.get(username);
    if(user == null) {
      throw new UsernameNotFoundException(username + " not found");
    }
    return new UWUserDetailsImpl(user);
  }

}
