/**
 * 
 */
package edu.wisc.uwss.preauth;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.web.authentication.preauth.AbstractPreAuthenticatedProcessingFilter;

import edu.wisc.uwss.UWUserDetailsImpl;

/**
 * Subclass of {@link AbstractPreAuthenticatedProcessingFilter} that returns {@link UWUserDetailsImpl}
 * for {@link #getPreAuthenticatedPrincipal(HttpServletRequest)}.
 * 
 * @author Nicholas Blair
 */
public class UWUserDetailsAuthenticationFilter extends
AbstractPreAuthenticatedProcessingFilter {

  private static Logger logger = LoggerFactory.getLogger(UWUserDetailsAuthenticationFilter.class);
  private String pviHeader = "wiscedupvi";
  private String usernameHeader = "uid";
  private String fullNameHeader = "cn";
  private String uddsHeader = "wisceduudds";
  private String emailAddressHeader = "mail";
  private String identityProviderHeader = "Shib-Identity-Provider";
  private String customLogoutPrefix = "/Shibboleth.sso/Logout?return=";
  private String customLogoutSuffix = "/logout/";
  /**
   * @param usernameHeader the usernameHeader to set
   */
  @Value("${preauth.usernameHeader:uid}")
  public void setUsernameHeader(String usernameHeader) {
    this.usernameHeader = usernameHeader;
  }
  /**
   * @param fullNameHeader the fullNameHeader to set
   */
  @Value("${preauth.fullnameHeader:cn}")
  public void setFullNameHeader(String fullNameHeader) {
    this.fullNameHeader = fullNameHeader;
  }
  /**
   * @param uddsHeader the uddsHeader to set
   */
  @Value("${preauth.uddsHeader:wisceduudds}")
  public void setUddsHeader(String uddsHeader) {
    this.uddsHeader = uddsHeader;
  }
  /**
   * @param emailAddressHeader the emailAddressHeader to set
   */
  @Value("${preauth.emailAddressHeader:mail}")
  public void setEmailAddressHeader(String emailAddressHeader) {
    this.emailAddressHeader = emailAddressHeader;
  }
  /**
   * @return the identityProviderHeader
   */
  public String getIdentityProviderHeader() {
    return identityProviderHeader;
  }
  /**
   * @param identityProviderHeader the identityProviderHeader to set
   */
  @Value("${preauth.identityProviderHeader:Shib-Identity-Provider}")
  public void setIdentityProviderHeader(String identityProviderHeader) {
    this.identityProviderHeader = identityProviderHeader;
  }
  /**
   * @return the customLogoutPath
   */
  public String getCustomLogoutPath() {
    return customLogoutSuffix;
  }
  /**
   * @param customLogoutSuffix the customLogoutSuffix to set
   */
  @Value("${preauth.customLogoutSuffix:/logout/}")
  public void setCustomLogoutSuffix(String customLogoutSuffix) {
    this.customLogoutSuffix = customLogoutSuffix;
  }
  /**
   * @return the customLogoutPrefix
   */
  public String getCustomLogoutPrefix() {
    return customLogoutPrefix;
  }
  /**
   * @param customLogoutPrefix the customLogoutPrefix to set
   */
  @Value("${preauth.customLogoutPrefix:/Shibboleth.sso/Logout?return=}")
  public void setCustomLogoutPrefix(String customLogoutPrefix) {
    this.customLogoutPrefix = customLogoutPrefix;
  }
  /**
   * @return the pviHeader
   */
  public String getPviHeader() {
    return pviHeader;
  }
  /**
   * @param pviHeader the pviHeader to set
   */
  @Value("${preauth.pviHeader:wiscedupvi}")
  public void setPviHeader(String pviHeader) {
    this.pviHeader = pviHeader;
  }
  /**
   * {@inheritDoc}
   * 
   * Returns a {@link UWUserDetailsImpl} from the attributes provided via request headers. Returns null
   * if the usernameHeader is not present.
   */
  @Override
  protected UWUserDetailsImpl getPreAuthenticatedPrincipal(HttpServletRequest request) {
    String uid = request.getHeader(usernameHeader);
    if(StringUtils.isBlank(uid)) {
      return null;
    }

    String pvi = request.getHeader(pviHeader);
    String cn = request.getHeader(fullNameHeader);
    Collection<String> uddsMembership = new ArrayList<>();
    Enumeration<String> uddsHeaders = request.getHeaders(uddsHeader);
    if(uddsHeaders != null) {
      uddsMembership = Collections.list(uddsHeaders);
    }
    String email = request.getHeader(emailAddressHeader);
    UWUserDetailsImpl result = new UWUserDetailsImpl(pvi, uid, "", cn, email, uddsMembership);
    result.setSource("preauth");

    String identityProvider = request.getHeader(identityProviderHeader);
    result.setCustomLogoutUrl(toCustomLogoutUrl(identityProvider));
    
    logger.debug("constructed {} from headers in request", result);
    return result;
  }
  /**
   * {@inheritDoc}
   * 
   * Returns an empty string as credentials (read: the password) are not available in our usage of Shibboleth.
   */
  @Override
  protected Object getPreAuthenticatedCredentials(HttpServletRequest request) {
    return "";
  }
  
  /**
   * 
   * @param identityProviderHeaderValue the value of the identityProviderHeader
   * @return the corresponding log out url, or null
   */
  protected String toCustomLogoutUrl(String identityProviderHeaderValue) {
    String result = null;
    if(StringUtils.isNotBlank(identityProviderHeaderValue)) {
      try {
        URL url = new URL(identityProviderHeaderValue);
        String path = url.getPath();
        if(StringUtils.isBlank(path)) {
          result = customLogoutPrefix + identityProviderHeaderValue + customLogoutSuffix;
        } else {
          result = customLogoutPrefix + identityProviderHeaderValue.replace(url.getPath(), customLogoutSuffix);
        }
        
      } catch (MalformedURLException e) {
        logger.debug("caught MalformedURLException for identityProviderHeader={}", identityProviderHeaderValue, e);
      }
    }
    logger.debug("value of {} was {} and transformed to customLogoutUrl of {}", identityProviderHeader, identityProviderHeaderValue, result);
    return result;
  }
}
