/**
 * 
 */
package edu.wisc.uwss.local;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.util.Properties;

import org.junit.Test;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import edu.wisc.uwss.UWUserDetailsImpl;
import edu.wisc.uwss.local.LocalUserDetailsServiceImpl;

/**
 * Tests for {@link LocalUserDetailsServiceImpl}.
 * 
 * @author Nicholas Blair
 */
public class LocalUserDetailsServiceImplTest {

	/**
	 * Verify {@link LocalUserDetailsServiceImpl#loadUserByUsername(String)} throws 
	 * {@link UsernameNotFoundException} for an unknown user.
	 */
	@Test(expected=UsernameNotFoundException.class)
	public void loadUserByUsername_notfound() {
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		service.loadUserByUsername("notthere");
	}
	
	/**
	 * We need to make sure we don't return the only instance of the 
	 * {@link UWUserDetailsImpl}, as Spring Security will mutate it.
	 */
	@Test
	public void loadUserByUsername_not_same_instance() {
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		UWUserDetailsImpl originalDemoUser = new UWUserDetailsImpl("UW000A000", "foo", "bar", "Foo Bar", "foo@foo.wisc.edu");
		service.addDemoUser(originalDemoUser);
		
		UserDetails retrieved = service.loadUserByUsername("foo");
		// .equals() should be true (usernames are the same)
		assertEquals(originalDemoUser, retrieved);
		// == MUST NOT be true (need to be 2 different instances
		assertFalse(retrieved == originalDemoUser);
		// verify repeated invocations are also new instances
		assertFalse(retrieved == service.loadUserByUsername("foo"));
	}
	/**
	 * Control experiment for {@link LocalUserDetailsServiceImpl}.
	 * Expects the "admin" user with "Amy Administrator" fullName.
	 * 
	 * @throws IOException 
	 */
	@Test
	public void setDemoUsers_control() throws IOException {
		Properties properties = new Properties();
		properties.load(new ClassPathResource("edu/wisc/uwss/local/local-users.properties").getInputStream());
		
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		service.setDemoUsers(properties);
		
		UWUserDetailsImpl userDetails = service.loadUserByUsername("admin");
		assertNotNull(userDetails);
		assertEquals("UW000A000", userDetails.getPvi());
		assertEquals("Amy Administrator", userDetails.getFullName());
		
		UWUserDetailsImpl jane = service.loadUserByUsername("jane");
		assertEquals("UW000A001", jane.getPvi());
        assertEquals("Jane Doe", jane.getFullName());
       
        UWUserDetailsImpl john = service.loadUserByUsername("john");
        assertEquals("UW000A002", john.getPvi());
        assertEquals("John Doe", john.getFullName());

        assertTrue(john.getUddsMembership().contains("A535005"));
	}
	/**
	 * Set up a properties instance including a user with a single UDDS.
	 * Verify that value properly set in {@link UWUserDetailsImpl#getUddsMembership()}.
	 */
	@Test
	public void setDemoUsers_with_no_udds_and_no_control() {
		Properties properties = new Properties();
		properties.put("test", "test,UW000A000,Nothing,foo@foo.wisc.edu");
		
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		service.setDemoUsers(properties);
		
		UWUserDetailsImpl userDetails = service.loadUserByUsername("test");
		assertNotNull(userDetails);
		assertEquals("UW000A000", userDetails.getPvi());
		assertEquals("Nothing", userDetails.getFullName());
		assertTrue(userDetails.getUddsMembership().isEmpty());
	}
	/**
	 * Set up a properties instance including a user with a single UDDS.
	 * Verify that value properly set in {@link UWUserDetailsImpl#getUddsMembership()}.
	 */
	@Test
	public void setDemoUsers_with_single_udds() {
		Properties properties = new Properties();
		properties.put("test", "test,UW000A000,Single UDDS,foo@foo.wisc.edu,A061234");
		
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		service.setDemoUsers(properties);
		
		UWUserDetailsImpl userDetails = service.loadUserByUsername("test");
		assertNotNull(userDetails);
		assertEquals("UW000A000", userDetails.getPvi());
		assertEquals("Single UDDS", ((UWUserDetailsImpl) userDetails).getFullName());
		assertTrue(((UWUserDetailsImpl) userDetails).getUddsMembership().contains("A061234"));
	}
	/**
	 * Set up a properties instance including a user with multiple UDDS values.
	 * Verify that values properly set in {@link UWUserDetailsImpl#getUddsMembership()}.
	 */
	@Test
	public void setDemoUsers_with_multiple_udds() {
		Properties properties = new Properties();
		properties.put("test", "test,UW000A000,Multiple UDDS,foo@foo.wisc.edu,A061234,A061235,A061236");
		
		LocalUserDetailsServiceImpl service = new LocalUserDetailsServiceImpl();
		service.setDemoUsers(properties);
		
		UWUserDetailsImpl userDetails = service.loadUserByUsername("test");
		assertNotNull(userDetails);
		assertEquals("UW000A000", userDetails.getPvi());
		assertEquals("Multiple UDDS", userDetails.getFullName());
		assertTrue(userDetails.getUddsMembership().contains("A061234"));
		assertTrue(userDetails.getUddsMembership().contains("A061235"));
		assertTrue(userDetails.getUddsMembership().contains("A061236"));
	}
}
