/**
 * 
 */
package edu.wisc.uwss.preauth;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.net.URL;
import java.util.Arrays;
import java.util.Collections;

import javax.servlet.http.HttpServletRequest;

import org.junit.Test;

import edu.wisc.uwss.UWUserDetailsImpl;

/**
 * Tests for {@link UWUserDetailsAuthenticationFilter}.
 * 
 * @author Nicholas Blair
 */
public class UWUserDetailsAuthenticationFilterTest {

	/**
	 * Control experiment for {@link UWUserDetailsAuthenticationFilter#getPreAuthenticatedPrincipal(HttpServletRequest)}.
	 */
	@Test
	public void getPreAuthenticatedPrincipal_control() {
		UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
		HttpServletRequest request = mock(HttpServletRequest.class);
		
		when(request.getHeader("uid")).thenReturn("bbadger");
		when(request.getHeader("cn")).thenReturn("Bucky Badger");
		when(request.getHeaders("wisceduudds")).thenReturn(Collections.enumeration(Arrays.asList("A061234")));
		when(request.getHeader("mail")).thenReturn("foo@foo.wisc.edu");
		UWUserDetailsImpl userDetails = filter.getPreAuthenticatedPrincipal(request);
		assertEquals("bbadger", userDetails.getUsername());
		assertEquals("Bucky Badger", userDetails.getFullName());
		assertEquals("foo@foo.wisc.edu", userDetails.getEmailAddress());
		assertEquals(1, userDetails.getUddsMembership().size());
		assertEquals("A061234", userDetails.getUddsMembership().iterator().next());
	}
	
	/**
	 * Verify behavior for {@link UWUserDetailsAuthenticationFilter#getPreAuthenticatedPrincipal(HttpServletRequest)}
	 * when no "udds" header is present. Example would be someone not employed by the University.
	 */
	@Test
	public void getPreAuthenticatedPrincipal_no_udds() {
		UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
		HttpServletRequest request = mock(HttpServletRequest.class);
		
		when(request.getHeader("uid")).thenReturn("bbadger");
		when(request.getHeader("cn")).thenReturn("Bucky Badger");
		UWUserDetailsImpl userDetails = filter.getPreAuthenticatedPrincipal(request);
		assertEquals("bbadger", userDetails.getUsername());
		assertEquals("Bucky Badger", userDetails.getFullName());
		assertTrue(userDetails.getUddsMembership().isEmpty());
	}
	/**
	 * Verify behavior for {@link UWUserDetailsAuthenticationFilter#getPreAuthenticatedPrincipal(HttpServletRequest)} when
	 * no "uid" header is present. This would represent a scenario where our pre-authentication (shib) environment isn't correctly
	 * configured; we want to make sure attempts to login would fail and not accidentially leak people in. 
	 */
	@Test
	public void getPreAuthenticatedPrincipal_no_uid() {
		UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
		HttpServletRequest request = mock(HttpServletRequest.class);
		assertNull(filter.getPreAuthenticatedPrincipal(request));
	}
	/**
	 * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
	 * null input.
	 */
	@Test
	public void toCustomLogoutUrl_null() {
	  UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
	  assertNull(filter.toCustomLogoutUrl(null));
	}
	/**
     * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * empty input.
     */
	@Test
    public void toCustomLogoutUrl_empty() {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      assertNull(filter.toCustomLogoutUrl(""));
    }
	/**
     * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * invalid input.
     */
	@Test
    public void toCustomLogoutUrl_invalid() {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      assertNull(filter.toCustomLogoutUrl("foo"));
    }
	/**
     * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * a valid input with no {@link URL#getPath()}.
     */
    @Test
    public void toCustomLogoutUrl_no_path() {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu"));
    }
	/**
     * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * expected valid input.
     */
	@Test
    public void toCustomLogoutUrl_success() {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
    }
	/**
	 * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * expected valid input and a different customLogoutPrefix.
	 */
	@Test
	public void toCustomLogoutUrl_alternate_prefix() {
	  UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
	  filter.setCustomLogoutPrefix("/Chibbolath.sso/Logout?return=");
      assertEquals("/Chibbolath.sso/Logout?return=https://somewhere.wisc.edu/logout/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
	}
	/**
     * Verify behavior of {@link UWUserDetailsAuthenticationFilter#toCustomLogoutUrl(String)} for
     * expected valid input and a different customLogoutSuffix.
     */
    @Test
    public void toCustomLogoutUrl_alternate_suffix() {
      UWUserDetailsAuthenticationFilter filter = new UWUserDetailsAuthenticationFilter();
      filter.setCustomLogoutSuffix("/luguot/");
      assertEquals("/Shibboleth.sso/Logout?return=https://somewhere.wisc.edu/luguot/", filter.toCustomLogoutUrl("https://somewhere.wisc.edu/idp/shibboleth"));
    }
}
